@@ -18,7 +18,9 @@ import operator
 import threading
 import warnings
 
+import debtcollector.removals as removals
 from oslo_config import cfg
+from oslo_utils import excutils
 
 from oslo_db import exception
 from oslo_db import options
@@ -133,6 +135,7 @@ class _TransactionFactory(object):
         self._engine_cfg = {
             'sqlite_fk': _Default(False),
             'mysql_sql_mode': _Default('TRADITIONAL'),
+            'mysql_enable_ndb': _Default(False),
             'idle_timeout': _Default(3600),
             'connection_debug': _Default(0),
             'max_pool_size': _Default(),
@@ -153,6 +156,7 @@ class _TransactionFactory(object):
         }
         self._transaction_ctx_cfg = {
             'rollback_reader_sessions': False,
+            'flush_on_subtransaction': False,
             }
         self._facade_cfg = {
             'synchronous_reader': True,
@@ -167,8 +171,7 @@ class _TransactionFactory(object):
                 'db_max_retries', 'db_inc_retry_interval',
                 'use_db_reconnect',
                 'db_retry_interval', 'min_pool_size',
-                'db_max_retry_interval',
-                'sqlite_db', 'backend'])
+                'db_max_retry_interval', 'backend'])
 
         self._started = False
         self._legacy_facade = None
@@ -195,6 +198,83 @@ class _TransactionFactory(object):
         :data:`oslo_config.cfg.CONF`, after applying the
         :data:`oslo_db.options.database_opts` configurational defaults to it.
 
+        :param connection: database URL
+
+        :param slave_connection: database URL
+
+        :param sqlite_fk: whether to enable SQLite foreign key pragma; default
+         False
+
+        :param mysql_sql_mode: MySQL SQL mode, defaults to TRADITIONAL
+
+        :param mysql_enable_ndb: enable MySQL Cluster (NDB) support
+
+        :param idle_timeout: connection pool recycle time,
+         defaults to 3600. Note the connection does not actually have to
+         be "idle" to be recycled.
+
+        :param connection_debug: engine logging level, defaults to 0. set to
+         50 for INFO, 100 for DEBUG.
+
+        :param max_pool_size: max size of connection pool, uses CONF for
+         default
+
+        :param max_overflow: max overflow for connection pool, uses CONF for
+         default
+
+        :param sqlite_synchronous: disable SQLite SYNCHRONOUS pragma if False;
+         defaults to True
+
+        :param connection_trace: enable tracing comments in logging
+
+        :param max_retries: max retries to connect, defaults to !0
+
+        :param retry_interval: time in seconds between retries, defaults to 10
+
+        :param thread_checkin: add sleep(0) on connection checkin to allow
+         greenlet yields, defaults to True
+
+        :param json_serializer: JSON serializer for PostgreSQL connections
+
+        :param json_deserializer: JSON deserializer for PostgreSQL connections
+
+        :param logging_name: logging name for engine
+
+        :param expire_on_commit: sets expire_on_commit for SQLAlchemy
+         sessionmaker; defaults to False
+
+        :param rollback_reader_sessions: if True, a :class:`.Session` object
+         will have its :meth:`.Session.rollback` method invoked at the end
+         of a ``@reader`` block, actively rolling back the transaction and
+         expiring the objects within, before the :class:`.Session` moves
+         on to be closed, which has the effect of releasing connection
+         resources back to the connection pool and detaching all objects.
+         If False, the :class:`.Session` is
+         not affected at the end of a ``@reader`` block; the underlying
+         connection referred to by this :class:`.Session` will still
+         be released in the enclosing context via the :meth:`.Session.close`
+         method, which still ensures that the DBAPI connection is rolled
+         back, however the objects associated with the :class:`.Session`
+         retain their database-persisted contents after they are detached.
+
+         .. seealso::
+
+            http://docs.sqlalchemy.org/en/rel_0_9/glossary.html#term-released\
+            SQLAlchemy documentation on what "releasing resources" means.
+
+        :param synchronous_reader: whether or not to assume a "reader" context
+         needs to guarantee it can read data committed by a "writer" assuming
+         replication lag is present; defaults to True.  When False, a
+         @reader context works the same as @async_reader and will select
+         the "slave" database if present.
+
+        :param flush_on_subtransaction: if True, a :class:`.Session` object
+         will have its :meth:`.Session.flush` method invoked whenever a context
+         manager or decorator that is not itself the originator of the top-
+         level or savepoint :class:`.Session` transaction exits - in this way
+         it behaves like a "subtransaction" from a :class:`.Session`
+         perspective.
+
         .. seealso::
 
             :meth:`._TransactionFactory.configure`
@@ -211,7 +291,8 @@ class _TransactionFactory(object):
         Behavior here is the same as that of
         :meth:`._TransactionFactory.configure_defaults`,
         with the exception that values specified here will **supersede** those
-        setup in the :class:`oslo_config.cfg.ConfigOpts` options.
+        setup in the :class:`oslo_config.cfg.ConfigOpts` options.  See
+        that method for a listing of all keyword arguments.
 
         .. seealso::
 
@@ -447,16 +528,25 @@ class _TestTransactionFactory(_TransactionFactory):
     within the global :class:`._TransactionContextManager`.
 
     """
-    def __init__(self, engine, maker, apply_global, synchronous_reader):
+
+    @removals.removed_kwarg(
+        'synchronous_reader',
+        'argument value is propagated from the parent _TransactionFactory')
+    def __init__(self, engine, maker, apply_global, from_factory=None, **kw):
+        # NOTE(zzzeek): **kw needed for backwards compability
         self._reader_engine = self._writer_engine = engine
         self._reader_maker = self._writer_maker = maker
         self._started = True
         self._legacy_facade = None
-        self.synchronous_reader = synchronous_reader
 
-        self._facade_cfg = _context_manager._factory._facade_cfg
-        self._transaction_ctx_cfg = \
-            _context_manager._factory._transaction_ctx_cfg
+        if from_factory is None:
+            from_factory = _context_manager._factory
+
+        self._facade_cfg = from_factory._facade_cfg
+        self._transaction_ctx_cfg = from_factory._transaction_ctx_cfg
+
+        self.synchronous_reader = self._facade_cfg['synchronous_reader']
+
         if apply_global:
             self.existing_factory = _context_manager._factory
             _context_manager._root_factory = self
@@ -468,9 +558,7 @@ class _TestTransactionFactory(_TransactionFactory):
 class _TransactionContext(object):
     """Represent a single database transaction in progress."""
 
-    def __init__(
-            self, factory, global_factory=None,
-            rollback_reader_sessions=False):
+    def __init__(self, factory, global_factory=None):
         """Construct a new :class:`.TransactionContext`.
 
         :param factory: the :class:`.TransactionFactory` which will
@@ -481,25 +569,6 @@ class _TransactionContext(object):
          objects created under this one.  When left as None the actual
          "global" factory is used.
 
-        :param rollback_reader_sessions: if True, a :class:`.Session` object
-         will have its :meth:`.Session.rollback` method invoked at the end
-         of a ``@reader`` block, actively rolling back the transaction and
-         expiring the objects within, before the :class:`.Session` moves
-         on to be closed, which has the effect of releasing connection
-         resources back to the connection pool and detaching all objects.
-         If False, the :class:`.Session` is
-         not affected at the end of a ``@reader`` block; the underlying
-         connection referred to by this :class:`.Session` will still
-         be released in the enclosing context via the :meth:`.Session.close`
-         method, which still ensures that the DBAPI connection is rolled
-         back, however the objects associated with the :class:`.Session`
-         retain their database-persisted contents after they are detached.
-
-         .. seealso::
-
-            http://docs.sqlalchemy.org/en/rel_0_9/glossary.html#term-released\
-            SQLAlchemy documentation on what "releasing resources" means.
-
         """
         self.factory = factory
         self.global_factory = global_factory
@@ -509,26 +578,30 @@ class _TransactionContext(object):
         self.transaction = None
         kw = self.factory._transaction_ctx_cfg
         self.rollback_reader_sessions = kw['rollback_reader_sessions']
+        self.flush_on_subtransaction = kw['flush_on_subtransaction']
 
     @contextlib.contextmanager
-    def _connection(self, savepoint=False):
+    def _connection(self, savepoint=False, context=None):
         if self.connection is None:
             try:
                 if self.session is not None:
                     # use existing session, which is outer to us
                     self.connection = self.session.connection()
                     if savepoint:
-                        with self.connection.begin_nested():
+                        with self.connection.begin_nested(), \
+                                self._add_context(self.connection, context):
                             yield self.connection
                     else:
-                        yield self.connection
+                        with self._add_context(self.connection, context):
+                            yield self.connection
                 else:
                     # is outermost
                     self.connection = self.factory._create_connection(
                         mode=self.mode)
                     self.transaction = self.connection.begin()
                     try:
-                        yield self.connection
+                        with self._add_context(self.connection, context):
+                            yield self.connection
                         self._end_connection_transaction(self.transaction)
                     except Exception:
                         self.transaction.rollback()
@@ -546,27 +619,26 @@ class _TransactionContext(object):
         else:
             # use existing connection, which is outer to us
             if savepoint:
-                with self.connection.begin_nested():
+                with self.connection.begin_nested(), \
+                        self._add_context(self.connection, context):
                     yield self.connection
             else:
-                yield self.connection
+                with self._add_context(self.connection, context):
+                    yield self.connection
 
     @contextlib.contextmanager
-    def _session(self, savepoint=False):
+    def _session(self, savepoint=False, context=None):
         if self.session is None:
             self.session = self.factory._create_session(
                 bind=self.connection, mode=self.mode)
             try:
                 self.session.begin()
-                yield self.session
+                with self._add_context(self.session, context):
+                    yield self.session
                 self._end_session_transaction(self.session)
             except Exception:
-                self.session.rollback()
-                # TODO(zzzeek) do we need save_and_reraise() here,
-                # or do newer eventlets not have issues?  we are using
-                # raw "raise" in many other places in oslo.db already
-                # (and one six.reraise()).
-                raise
+                with excutils.save_and_reraise_exception():
+                    self.session.rollback()
             finally:
                 self.session.close()
                 self.session = None
@@ -574,9 +646,20 @@ class _TransactionContext(object):
             # use existing session, which is outer to us
             if savepoint:
                 with self.session.begin_nested():
-                    yield self.session
+                    with self._add_context(self.session, context):
+                        yield self.session
             else:
-                yield self.session
+                with self._add_context(self.session, context):
+                    yield self.session
+                if self.flush_on_subtransaction:
+                    self.session.flush()
+
+    @contextlib.contextmanager
+    def _add_context(self, connection, context):
+        restore_context = connection.info.get('using_context')
+        connection.info['using_context'] = context
+        yield connection
+        connection.info['using_context'] = restore_context
 
     def _end_session_transaction(self, session):
         if self.mode is _WRITER:
@@ -595,7 +678,8 @@ class _TransactionContext(object):
         else:
             transaction.rollback()
 
-    def _produce_block(self, mode, connection, savepoint, allow_async=False):
+    def _produce_block(self, mode, connection, savepoint, allow_async=False,
+                       context=None):
         if mode is _WRITER:
             self._writer()
         elif mode is _ASYNC_READER:
@@ -603,9 +687,9 @@ class _TransactionContext(object):
         else:
             self._reader(allow_async)
         if connection:
-            return self._connection(savepoint)
+            return self._connection(savepoint, context=context)
         else:
-            return self._session(savepoint)
+            return self._session(savepoint, context=context)
 
     def _writer(self):
         if self.mode is None:
@@ -803,6 +887,8 @@ class _TransactionContextManager(object):
         """
 
         existing_factory = self._factory
+        if not existing_factory._started:
+            existing_factory._start()
         maker = existing_factory._writer_maker
         maker_kwargs = existing_factory._maker_args_for_conf(cfg.CONF)
         maker = orm.get_maker(engine=engine, **maker_kwargs)
@@ -810,8 +896,7 @@ class _TransactionContextManager(object):
         factory = _TestTransactionFactory(
             engine, maker,
             apply_global=False,
-            synchronous_reader=existing_factory.
-            _facade_cfg['synchronous_reader']
+            from_factory=existing_factory
         )
         return self.patch_factory(factory)
 
@@ -931,9 +1016,7 @@ class _TransactionContextManager(object):
 
         if current is None:
             current = transaction_contexts_by_thread.current = \
-                _TransactionContext(
-                    use_factory, global_factory=global_factory,
-                    **use_factory._transaction_ctx_cfg)
+                _TransactionContext(use_factory, global_factory=global_factory)
 
         try:
             if self._mode is not None:
@@ -941,7 +1024,8 @@ class _TransactionContextManager(object):
                     mode=self._mode,
                     connection=self._connection,
                     savepoint=self._savepoint,
-                    allow_async=self._allow_async) as resource:
+                    allow_async=self._allow_async,
+                    context=context) as resource:
                     yield resource
             else:
                 yield
@@ -1109,6 +1193,9 @@ class LegacyEngineFacade(object):
 
     :keyword mysql_sql_mode: the SQL mode to be used for MySQL sessions.
                              (defaults to TRADITIONAL)
+    :keyword mysql_enable_ndb: If True, transparently enables support for
+                               handling MySQL Cluster (NDB).
+                               (defaults to False)
     :keyword idle_timeout: timeout before idle sql connections are reaped
                            (defaults to 3600)
     :keyword connection_debug: verbosity of SQL debugging information.
