@@ -19,7 +19,6 @@ import sys
 from sqlalchemy import event
 from sqlalchemy import exc as sqla_exc
 
-from oslo_db._i18n import _LE
 from oslo_db import exception
 
 
@@ -396,6 +395,11 @@ def _is_db_connection_error(operational_error, match, engine_name,
     raise exception.DBConnectionError(operational_error)
 
 
+@filters("*", sqla_exc.NotSupportedError, r".*")
+def _raise_for_NotSupportedError(error, match, engine_name, is_disconnect):
+    raise exception.DBNotSupportedError(error)
+
+
 @filters("*", sqla_exc.DBAPIError, r".*")
 def _raise_for_remaining_DBAPIError(error, match, engine_name, is_disconnect):
     """Filter for remaining DBAPIErrors.
@@ -407,7 +411,7 @@ def _raise_for_remaining_DBAPIError(error, match, engine_name, is_disconnect):
         raise exception.DBConnectionError(error)
     else:
         LOG.exception(
-            _LE('DBAPIError exception wrapped from %s') % error)
+            'DBAPIError exception wrapped from %s' % error)
         raise exception.DBError(error)
 
 
@@ -418,7 +422,7 @@ def _raise_for_unicode_encode(error, match, engine_name, is_disconnect):
 
 @filters("*", Exception, r".*")
 def _raise_for_all_others(error, match, engine_name, is_disconnect):
-    LOG.exception(_LE('DB exception wrapped.'))
+    LOG.exception('DB exception wrapped.')
     raise exception.DBError(error)
 
 ROLLBACK_CAUSE_KEY = 'oslo.db.sp_rollback_cause'
@@ -481,7 +485,11 @@ def register_engine(engine):
     def rollback_savepoint(conn, name, context):
         exc_info = sys.exc_info()
         if exc_info[1]:
-            conn.info[ROLLBACK_CAUSE_KEY] = exc_info[1]
+            # NOTE(zzzeek) accessing conn.info on an invalidated
+            # connection causes it to reconnect, which we don't
+            # want to do inside a rollback handler
+            if not conn.invalidated:
+                conn.info[ROLLBACK_CAUSE_KEY] = exc_info[1]
         # NOTE(zzzeek) this eliminates a reference cycle between tracebacks
         # that would occur in Python 3 only, which has been shown to occur if
         # this function were in fact part of the traceback.  That's not the
@@ -493,7 +501,11 @@ def register_engine(engine):
     @event.listens_for(engine, "rollback")
     @event.listens_for(engine, "commit")
     def pop_exc_tx(conn):
-        conn.info.pop(ROLLBACK_CAUSE_KEY, None)
+        # NOTE(zzzeek) accessing conn.info on an invalidated
+        # connection causes it to reconnect, which we don't
+        # want to do inside a rollback handler
+        if not conn.invalidated:
+            conn.info.pop(ROLLBACK_CAUSE_KEY, None)
 
     # .. as well as connection pool checkin (just in case).
     # the .info dictionary lasts as long as the DBAPI connection itself
