@@ -14,7 +14,6 @@
 # under the License.
 
 import fixtures
-import logging
 import os
 import testresources
 import testscenarios
@@ -25,9 +24,6 @@ from oslo_db.sqlalchemy import provision
 from oslo_db.sqlalchemy import utils
 
 
-LOG = logging.getLogger(__name__)
-
-
 class ReplaceEngineFacadeFixture(fixtures.Fixture):
     """A fixture that will plug the engine of one enginefacade into another.
 
