@@ -38,14 +38,15 @@ from sqlalchemy import Index
 from sqlalchemy import inspect
 from sqlalchemy import Integer
 from sqlalchemy import MetaData
+from sqlalchemy.sql.expression import cast
 from sqlalchemy.sql.expression import literal_column
 from sqlalchemy.sql import text
 from sqlalchemy import String
 from sqlalchemy import Table
 from sqlalchemy.types import NullType
 
+from oslo_db._i18n import _
 from oslo_db import exception
-from oslo_db._i18n import _, _LI, _LW
 from oslo_db.sqlalchemy import models
 
 # NOTE(ochuprykov): Add references for backwards compatibility
@@ -154,6 +155,11 @@ def paginate_query(query, model, limit, sort_keys, marker=None,
     marker, then the actual marker object must be fetched from the db and
     passed in to us as marker.
 
+    The "offset" parameter is intentionally avoided. As offset requires a
+    full scan through the preceding results each time, criteria-based
+    pagination is preferred. See http://use-the-index-luke.com/no-offset
+    for further background.
+
     :param query: the query object to which we should add paging/sorting
     :param model: the ORM model class
     :param limit: maximum number of items to return
@@ -169,8 +175,8 @@ def paginate_query(query, model, limit, sort_keys, marker=None,
     :return: The query with sorting/pagination added.
     """
     if _stable_sorting_order(model, sort_keys) is False:
-        LOG.warning(_LW('Unique keys not in sort_keys. '
-                        'The sorting order may be unstable.'))
+        LOG.warning('Unique keys not in sort_keys. '
+                    'The sorting order may be unstable.')
 
     assert(not (sort_dir and sort_dirs))
 
@@ -236,11 +242,15 @@ def paginate_query(query, model, limit, sort_keys, marker=None,
                     crit_attrs.append((model_attr == marker_values[j]))
 
                 model_attr = getattr(model, sort_keys[i])
+                val = marker_values[i]
+                # sqlalchemy doesn't like booleans in < >. bug/1656947
+                if isinstance(model_attr.type, Boolean):
+                    val = int(val)
+                    model_attr = cast(model_attr, Integer)
                 if sort_dirs[i].startswith('desc'):
-                    crit_attrs.append((model_attr < marker_values[i]))
+                    crit_attrs.append((model_attr < val))
                 else:
-                    crit_attrs.append((model_attr > marker_values[i]))
-
+                    crit_attrs.append((model_attr > val))
                 criteria = sqlalchemy.sql.and_(*crit_attrs)
                 criteria_list.append(criteria)
 
@@ -514,8 +524,8 @@ def drop_old_duplicate_entries_from_table(engine, table_name,
         rows_to_delete_select = sqlalchemy.sql.select(
             [table.c.id]).where(delete_condition)
         for row in engine.execute(rows_to_delete_select).fetchall():
-            LOG.info(_LI("Deleting duplicated row with id: %(id)s from table: "
-                         "%(table)s"), dict(id=row[0], table=table_name))
+            LOG.info("Deleting duplicated row with id: %(id)s from table: "
+                     "%(table)s", dict(id=row[0], table=table_name))
 
         if use_soft_delete:
             delete_statement = table.update().\
@@ -1129,6 +1139,36 @@ def get_non_innodb_tables(connectable, skip_tables=('migrate_version',
     return [i[0] for i in noninnodb]
 
 
+def get_non_ndbcluster_tables(connectable, skip_tables=None):
+    """Get a list of tables which don't use MySQL Cluster (NDB) storage engine.
+
+    :param connectable: a SQLAlchemy Engine or Connection instance
+    :param skip_tables: a list of tables which might have a different
+                        storage engine
+    """
+    query_str = """
+        SELECT table_name
+        FROM information_schema.tables
+        WHERE table_schema = :database AND
+              engine != 'ndbcluster'
+    """
+
+    params = {}
+    if skip_tables:
+        params = dict(
+            ('skip_%s' % i, table_name)
+            for i, table_name in enumerate(skip_tables)
+        )
+
+        placeholders = ', '.join(':' + p for p in params)
+        query_str += ' AND table_name NOT IN (%s)' % placeholders
+
+    params['database'] = connectable.engine.url.database
+    query = text(query_str)
+    nonndbcluster = connectable.execute(query, **params)
+    return [i[0] for i in nonndbcluster]
+
+
 class NonCommittingConnectable(object):
     """A ``Connectable`` substitute which rolls all operations back.
 
