@@ -13,6 +13,7 @@
 import collections
 import contextlib
 import copy
+import fixtures
 import pickle
 import warnings
 
@@ -24,6 +25,7 @@ from sqlalchemy import Column
 from sqlalchemy import Integer
 from sqlalchemy import MetaData
 from sqlalchemy.orm import mapper
+from sqlalchemy.orm import Session
 from sqlalchemy import select
 from sqlalchemy import String
 from sqlalchemy import Table
@@ -55,6 +57,7 @@ class SingletonConnection(SingletonOnName):
     def __init__(self, **kw):
         super(SingletonConnection, self).__init__(
             "connection", **kw)
+        self.info = {}
 
 
 class SingletonEngine(SingletonOnName):
@@ -113,14 +116,16 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
         writer_conn = SingletonConnection()
         writer_engine = SingletonEngine(writer_conn)
         writer_session = mock.Mock(
-            connection=mock.Mock(return_value=writer_conn))
+            connection=mock.Mock(return_value=writer_conn),
+            info={})
         writer_maker = mock.Mock(return_value=writer_session)
 
         if self.slave_uri:
             async_reader_conn = SingletonConnection()
             async_reader_engine = SingletonEngine(async_reader_conn)
             async_reader_session = mock.Mock(
-                connection=mock.Mock(return_value=async_reader_conn))
+                connection=mock.Mock(return_value=async_reader_conn),
+                info={})
             async_reader_maker = mock.Mock(return_value=async_reader_session)
 
         else:
@@ -389,6 +394,12 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
             makers, enginefacade._WRITER,
             connection, assert_calls)
 
+    def _emit_sub_writer_session(self, session):
+        return self._emit_sub_session(enginefacade._WRITER, session)
+
+    def _emit_sub_reader_session(self, session):
+        return self._emit_sub_session(enginefacade._READER, session)
+
     @contextlib.contextmanager
     def _assert_session(
             self, makers, writer, connection=None, assert_calls=True):
@@ -418,6 +429,13 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
                 session.mock_calls,
                 self.sessions.element_for_writer(writer).mock_calls)
 
+    @contextlib.contextmanager
+    def _emit_sub_session(self, writer, session):
+        yield session
+        if enginefacade._context_manager.\
+                _factory._transaction_ctx_cfg['flush_on_subtransaction']:
+            session.flush()
+
     def test_dispose_pool(self):
         facade = enginefacade.transaction_context()
 
@@ -757,6 +775,20 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
                 with self._assert_reader_session(makers) as session:
                     session.execute("test1")
 
+    def test_using_context_present_in_session_info(self):
+        context = oslo_context.RequestContext()
+
+        with enginefacade.reader.using(context) as session:
+            self.assertEqual(context, session.info['using_context'])
+        self.assertIsNone(session.info['using_context'])
+
+    def test_using_context_present_in_connection_info(self):
+        context = oslo_context.RequestContext()
+
+        with enginefacade.writer.connection.using(context) as connection:
+            self.assertEqual(context, connection.info['using_context'])
+        self.assertIsNone(connection.info['using_context'])
+
     def test_using_reader_rollback_reader_session(self):
         enginefacade.configure(rollback_reader_sessions=True)
 
@@ -771,6 +803,23 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
                 with self._assert_reader_session(makers) as session:
                     session.execute("test1")
 
+    def test_using_flush_on_nested(self):
+        enginefacade.configure(flush_on_nested=True)
+
+        context = oslo_context.RequestContext()
+
+        with enginefacade.writer.using(context) as session:
+            with enginefacade.writer.using(context) as session:
+                self._assert_ctx_session(context, session)
+                session.execute("test1")
+
+        with self._assert_engines() as engines:
+            with self._assert_makers(engines) as makers:
+                with self._assert_writer_session(makers) as session:
+                    with self._emit_sub_writer_session(
+                            session) as session:
+                        session.execute("test1")
+
     def test_using_writer(self):
         context = oslo_context.RequestContext()
 
@@ -1160,7 +1209,10 @@ class PatchFactoryTest(oslo_test_base.BaseTestCase):
 
     def test_patch_engine(self):
         normal_mgr = enginefacade.transaction_context()
-        normal_mgr.configure(connection="sqlite:///foo.db")
+        normal_mgr.configure(
+            connection="sqlite:///foo.db",
+            rollback_reader_sessions=True
+        )
 
         @normal_mgr.writer
         def go1(context):
@@ -1179,6 +1231,17 @@ class PatchFactoryTest(oslo_test_base.BaseTestCase):
                 s1.bind.url,
                 "sqlite:///bar.db")
 
+            self.assertTrue(
+                enginefacade._transaction_ctx_for_context(
+                    context).rollback_reader_sessions
+            )
+
+            # ensure this defaults to True
+            self.assertTrue(
+                enginefacade._transaction_ctx_for_context(
+                    context).factory.synchronous_reader
+            )
+
         def create_engine(sql_connection, **kw):
             return mock.Mock(url=sql_connection)
 
@@ -1195,6 +1258,40 @@ class PatchFactoryTest(oslo_test_base.BaseTestCase):
             reset()
             go1(context)
 
+    def test_patch_not_started(self):
+        normal_mgr = enginefacade.transaction_context()
+        normal_mgr.configure(
+            connection="sqlite:///foo.db",
+            rollback_reader_sessions=True
+        )
+
+        @normal_mgr.writer
+        def go1(context):
+            s1 = context.session
+
+            self.assertEqual(
+                s1.bind.url,
+                "sqlite:///bar.db")
+
+            self.assertTrue(
+                enginefacade._transaction_ctx_for_context(
+                    context).rollback_reader_sessions
+            )
+
+        def create_engine(sql_connection, **kw):
+            return mock.Mock(url=sql_connection)
+
+        with mock.patch(
+                "oslo_db.sqlalchemy.engines.create_engine", create_engine):
+            mock_engine = create_engine("sqlite:///bar.db")
+
+            context = oslo_context.RequestContext()
+            reset = normal_mgr.patch_engine(mock_engine)
+            go1(context)
+            self.assertIs(
+                normal_mgr._factory._writer_engine, mock_engine)
+            reset()
+
     def test_new_manager_from_config(self):
         normal_mgr = enginefacade.transaction_context()
         normal_mgr.configure(
@@ -1507,6 +1604,7 @@ class LiveFacadeTest(test_base.DbTestCase):
             'user', metadata,
             Column('id', Integer, primary_key=True),
             Column('name', String(30)),
+            Column('favorite_color', String(10), default='yellow'),
             mysql_engine='InnoDB'
         )
         self.user_table = user_table
@@ -1558,6 +1656,57 @@ class LiveFacadeTest(test_base.DbTestCase):
             session.query(self.User.name).scalar()
         )
 
+    @mock.patch.object(Session, 'commit')
+    @mock.patch.object(Session, 'rollback')
+    def test_save_and_reraise_when_rollback_exception(self,
+                                                      rollback_patch,
+                                                      commit_patch):
+        context = oslo_context.RequestContext()
+        log = self.useFixture(fixtures.FakeLogger())
+
+        class RollbackException(Exception):
+            pass
+
+        class CommitException(Exception):
+            pass
+
+        commit_patch.side_effect = CommitException()
+        rollback_patch.side_effect = RollbackException()
+
+        @enginefacade.writer
+        def go_session(context):
+            context.session.add(self.User(name="u1"))
+
+        self.assertRaises(RollbackException, go_session, context)
+        self.assertIn('CommitException', log.output)
+
+    def test_flush_on_subtransaction(self):
+        facade = enginefacade.transaction_context()
+        facade.configure(
+            connection=self.engine.url,
+            flush_on_subtransaction=True)
+        facade.patch_engine(self.engine)
+        context = oslo_context.RequestContext()
+
+        with facade.writer.using(context):
+            with facade.writer.using(context):
+                u = self.User(name="u1")
+                context.session.add(u)
+            self.assertEqual('yellow', u.favorite_color)
+
+    def test_flush_on_subtransaction_default_off(self):
+        context = oslo_context.RequestContext()
+        facade = enginefacade.transaction_context()
+        facade.configure(connection=self.engine.url)
+        facade.patch_engine(self.engine)
+
+        with facade.writer.using(context):
+            with facade.writer.using(context):
+                u = self.User(name="u1")
+                context.session.add(u)
+            self.assertIsNone(u.favorite_color)
+        self.assertEqual('yellow', u.favorite_color)
+
     def test_context_deepcopy_on_session(self):
         context = oslo_context.RequestContext()
         with enginefacade.writer.using(context) as session:
